// RTTY Transmitter from AD9851
// George Smart, M1GEO.
//  here: http://www,george-smart.co.uk/wiki/Arduino_RTTY
//
// AD9851 Code
// Originally by Peter Marks
//  here: http://blog.marxy.org/2008/05/controlling-ad9851-dds-with-arduino.html
//
// RTTY Baudot Code
// Tim Zaman
//  here: http://www.timzaman.nl/?p=138&lang=en

// DDS Reference Oscilliator Frequency, in Hz. (Remember, the PLL).
#define DDS_REF 180000000

// RTTY Output Frequency
#define RTTY_TXF  50.100e6

// RTTY Offset Frequency
#define RTTY_OSET 170

// DDS/Arduino Connections
#define DDS_LOAD  8
#define DDS_CLOCK 9
#define DDS_DATA  10
#define LED       13

#include <stdint.h>

#define ARRAY_LEN 32
#define LETTERS_SHIFT 31
#define FIGURES_SHIFT 27
#define LINEFEED 2
#define CARRRTN  8
 
#define is_lowercase(ch)    ((ch) >= 'a' && (ch) <= 'z')
#define is_uppercase(ch)    ((ch) >= 'A' && (ch) <= 'Z')
 
unsigned long time;
 
char letters_arr[33] = "\000E\nA SIU\rDRJNFCKTZLWHYPQOBG\000MXV\000";
char figures_arr[33] = "\0003\n- \a87\r$4',!:(5\")2#6019?&\000./;\000";

void setup()
{
  // Set all pins to output states
  pinMode (DDS_DATA,  OUTPUT);
  pinMode (DDS_CLOCK, OUTPUT);
  pinMode (DDS_LOAD,  OUTPUT);
  pinMode (LED,       OUTPUT);
  
  // Setup RS232
  Serial.begin(9600);
}

void loop()
{
  
  // let the radio settle a bit before we start sending data
  delay(100);
  frequency(RTTY_TXF);
  delay(1000);
  
  time=millis();
  
  rtty_txstring("\r\n      \r\n");
  rtty_txstring("George Smart, M1GEO.\r\n");
  rtty_txstring("http://www.george-smart.co.uk/\r\n");
  rtty_txstring("Arduino RTTY Test!\r\n\r\n");
  rtty_txstring("\"An expert is a person who has made all the mistakes that can be made in a very narrow field\"  Nils Bohr\r\n\r\n");
  rtty_txstring("\"Most of the important things in the world have been accomplished by people who have kept on trying when there seemed to be no hope at all\"  Dale Carnegie\r\n\r\n");
  rtty_txstring("\"If others would think as hard as I did, then they would get similar results\"  Newton\r\n\r\n");
  rtty_txstring("\"If you can’t explain what you are doing to a nine-year-old, then either you still don’t understand it very well, or it’s not all that worthwhile in the first place.\"  Albert Einstein\r\n\r\n");
  rtty_txstring("-----\r\n");
  
  
  time=millis()-time;
  Serial.println(time);

  digitalWrite (LED, HIGH); 
  delay(100);
  digitalWrite (LED, LOW); 
  delay(1000);
  
  frequency(0);
  delay(1000);

}

void frequency(unsigned long frequency) {
  unsigned long tuning_word = (frequency * pow(2, 32)) / DDS_REF;
  digitalWrite (DDS_LOAD, LOW); // take load pin low

  for(int i = 0; i < 32; i++) {
    if ((tuning_word & 1) == 1)
      outOne();
    else
      outZero();
    tuning_word = tuning_word >> 1;
  }
  byte_out(0x09);
  digitalWrite (DDS_LOAD, HIGH); // Take load pin high again
}

void byte_out(unsigned char byte) {
  int i;

  for (i = 0; i < 8; i++) {
    if ((byte & 1) == 1)
      outOne();
    else
      outZero();
    byte = byte >> 1;
  }
}

void outOne() {
  digitalWrite(DDS_CLOCK, LOW);
  digitalWrite(DDS_DATA, HIGH);
  digitalWrite(DDS_CLOCK, HIGH);
  digitalWrite(DDS_DATA, LOW);
}

void outZero() {
  digitalWrite(DDS_CLOCK, LOW);
  digitalWrite(DDS_DATA, LOW);
  digitalWrite(DDS_CLOCK, HIGH);
}

uint8_t char_to_baudot(char c, char *array)
{
  int i;
  for (i = 0; i < ARRAY_LEN; i++)
  {
    if (array[i] == c)
      return i;
  }
 
  return 0;
}
 
void rtty_txbyte(uint8_t b)
{
  int8_t i;
 
  rtty_txbit(0);
 
  /* TODO: I don't know if baudot is MSB first or LSB first */
  /* for (i = 4; i >= 0; i--) */
  for (i = 0; i < 5; i++)
  {
    if (b & (1 << i))
      rtty_txbit(1);
    else
      rtty_txbit(0);
  }
 
  rtty_txbit(1);
}
 
enum baudot_mode
{
  NONE,
  LETTERS,
  FIGURES
};
 
void rtty_txstring(char *str)
{
  enum baudot_mode current_mode = NONE;
  char c;
  uint8_t b;
 
  while (*str != '\0')
  {
    c = *str;
    /* some characters are available in both sets */
    if (c == '\n')
    {
      rtty_txbyte(LINEFEED);
    }
    else if (c == '\r')
    {
      rtty_txbyte(CARRRTN);
    }
    else if (is_lowercase(*str) || is_uppercase(*str))
    {
      if (is_lowercase(*str))
      {
        c -= 32;
      }
 
      if (current_mode != LETTERS)
      {
        rtty_txbyte(LETTERS_SHIFT);
        current_mode = LETTERS;
      }
 
      rtty_txbyte(char_to_baudot(c, letters_arr));
    }
    else
    {
      b = char_to_baudot(c, figures_arr);
 
      if (b != 0 && current_mode != FIGURES)
      {
        rtty_txbyte(FIGURES_SHIFT);
        current_mode = FIGURES;
      }
 
      rtty_txbyte(b);
    }
 
    str++;
  }
}
 
 
 
// Transmit a bit as a mark or space
void rtty_txbit (int bit) {
  if (bit) {
    // High - mark
    //digitalWrite(2, HIGH);
    //digitalWrite(3, LOW);
    
    frequency(RTTY_TXF + RTTY_OSET);
    
  } else {
    // Low - space
    //digitalWrite(3, HIGH);
    //digitalWrite(2, LOW);
    
    frequency(RTTY_TXF);
    
  }
  
  // Delay appropriately - tuned to 45.45 baud.
  delay(22); //sets the baud rate
  //delayMicroseconds(250);
}
